/* *********************************  */ 
/*
Program name:	Cable_centerline1.c		
Program description:	Creates an extended circular section body centerline	
Release date:			January 30, 2009
Version:				Initial release
Created by:				Frank Swinkels, frank.swinkels@plma.com.au

I provide programming code examples from which you can generate similar function tailored to your
own specific needs.  The code is provided for illustrative purposes only. The example has not been
thoroughly tested under all conditions. We therefore, cannot guarantee or imply reliability,
serviceability, or function of the program.  The code contained herein is provided to you "AS IS" 
without any warranties of any kind. The implied warranties of fitness for a particular purpose is
expressly disclaimed.  */

#include <stdio.h>
#include <string.h>
#include <math.h>

#include <uf.h>
#include <uf_disp.h>
#include <uf_obj.h>
#include <uf_ui.h>
#include <uf_object_types.h>
#include <uf_curve.h>
#include <uf_modl.h>
#include <uf_vec.h>



#define UF_CALL(X) (report_error( __FILE__, __LINE__, #X, (X)))

static int report_error( char *file, int line, char *call, int irc)
{
    if (irc)
    {
        char err[133],
             msg[133];

        sprintf(msg, "*** ERROR code %d at line %d in %s:\n+++ ",
            irc, line, file);
        UF_get_fail_message(irc, err);

    /*  NOTE:  UF_print_syslog is new in V18 */

        UF_print_syslog(msg, FALSE);
        UF_print_syslog(err, FALSE);
        UF_print_syslog("\n", FALSE);
        UF_print_syslog(call, FALSE);
        UF_print_syslog(";\n", FALSE);

        if (!UF_UI_open_listing_window())
        {
            UF_UI_write_listing_window(msg);
            UF_UI_write_listing_window(err);
            UF_UI_write_listing_window("\n");
            UF_UI_write_listing_window(call);
            UF_UI_write_listing_window(";\n");
        }
    }

    return(irc);
}

static int mask_for_faces(UF_UI_selection_p_t select, void *type)
{
    UF_UI_mask_t
        mask = { UF_solid_type, 0, UF_UI_SEL_FEATURE_ANY_FACE };

    if (!UF_CALL(UF_UI_set_sel_mask(select,
            UF_UI_SEL_MASK_CLEAR_AND_ENABLE_SPECIFIC, 1, &mask)))
        return (UF_UI_SEL_SUCCESS);
    else
        return (UF_UI_SEL_FAILURE);
}

static tag_t select_a_face(char *prompt)
{
    int
        resp;
    double
        cp[3];
    tag_t
        object,
        view;

    UF_CALL(UF_UI_select_with_single_dialog(prompt, "",
        UF_UI_SEL_SCOPE_ANY_IN_ASSEMBLY, mask_for_faces, NULL, &resp,
        &object, cp, &view));

    if (resp == UF_UI_OBJECT_SELECTED || resp == UF_UI_OBJECT_SELECTED_BY_NAME)
    {
        UF_CALL(UF_DISP_set_highlight(object, FALSE));
        return object;
    }
    else return NULL_TAG;

}


static void do_it(void)
{
	int		face_type=2, ii, jj, style1, no_pts, pos1, err, resp;
	int		degree3 = 3, periodicity = 0;

	double	*v_params, toler=0.001, magnitude, length1[1]={1.0};
	double	param[4][2], pnts[4][3], junk3[3], vecv[4][3];
	double	cpnt1[200][3], cvect1[200][3];
	double	temp_vec[3];

	char	menu[1][16]={"Length"};

	tag_t	face = NULL_TAG, temp_tag;

	UF_MODL_bsurface_t		bsurf;

	UF_CURVE_pt_slope_crvatr_t	*point_data1;

	point_data1 = (UF_CURVE_pt_slope_crvatr_t *)
				UF_allocate_memory(200 * sizeof(UF_CURVE_pt_slope_crvatr_t),&err);

start1: ;

	resp = uc1609("Enter centerline extension", menu, 1, &length1[0], &err);

	if ((resp== 1) || (resp == 2)) goto ending1;

	face = select_a_face("Select circular face");


		UF_CALL(UF_MODL_ask_bsurf(face, &bsurf));

		// test for closed in u direction or in v direction

		if ((bsurf.knots_u[0] < 0.0) && (bsurf.knots_v[0]== 0.0))
		{
			style1 = 1;
			no_pts = bsurf.num_poles_v/(bsurf.order_v-2);
			v_params = UF_allocate_memory( no_pts * sizeof(double),&err);
			pos1 = bsurf.order_v-1;
			for (ii=0;ii<no_pts;ii++)
			{
				v_params[ii] = bsurf.knots_v[pos1+ii*2];
			}
		}
		/*	else if ((bsurf.knots_u[0] == 0.0) && (bsurf.knots_v[0] < 0.0))
		{
		style1 = 2;
		}  */
		else
		{
			uc1601 ("Selected face not supported", 1);
			goto ending1;
		}

		if (style1 == 1)
		{
			param[0][0]=0.0;
			param[1][0]=0.25;
			param[2][0]=0.5;
			param[3][0]=0.75;
			for (ii=0;ii<no_pts;ii++)
			{
				for (jj=0;jj<4;jj++)
				{
					param[jj][1]=v_params[ii];
					UF_MODL_ask_face_props(face, &param[jj][0], &pnts[jj][0], junk3,&vecv[jj][0],
						junk3,junk3,junk3,junk3);
					UF_VEC3_unitize(&vecv[jj][0], toler, &magnitude, &vecv[jj][0]);
				}
				UF_VEC3_add(&pnts[0][0],&pnts[1][0], &cpnt1[ii+1][0]);
				UF_VEC3_add(&pnts[2][0],&cpnt1[ii+1][0], &cpnt1[ii+1][0]);
				UF_VEC3_add(&pnts[3][0],&cpnt1[ii+1][0], &cpnt1[ii+1][0]);
				cpnt1[ii+1][0] /= 4.0;
				cpnt1[ii+1][1] /= 4.0;
				cpnt1[ii+1][2] /= 4.0;

				UF_VEC3_add(&vecv[0][0],&vecv[1][0], &cvect1[ii+1][0]);
				UF_VEC3_add(&vecv[2][0],&cvect1[ii+1][0], &cvect1[ii+1][0]);
				UF_VEC3_add(&vecv[3][0],&cvect1[ii+1][0], &cvect1[ii+1][0]);
				UF_VEC3_unitize(&cvect1[ii+1][0], toler, &magnitude, &cvect1[ii+1][0]);
			}
			cvect1[0][0] = cvect1[1][0];
			cvect1[0][1] = cvect1[1][1];
			cvect1[0][2] = cvect1[1][2];
			cvect1[no_pts+1][0] = cvect1[no_pts][0];
			cvect1[no_pts+1][1] = cvect1[no_pts][1];
			cvect1[no_pts+1][2] = cvect1[no_pts][2];

			UF_VEC3_negate(&cvect1[0][0], temp_vec);
			UF_VEC3_scale(length1[0],temp_vec,temp_vec);
			UF_VEC3_add(&cpnt1[1][0],temp_vec, &cpnt1[0][0]);

			UF_VEC3_scale(length1[0],&cvect1[no_pts][0],temp_vec);
			UF_VEC3_add(&cpnt1[no_pts][0],temp_vec, &cpnt1[no_pts+1][0]);

			for (ii=0;ii<(no_pts+2);ii++)
			{
				point_data1[ii].point[0] = cpnt1[ii][0];
				point_data1[ii].point[1] = cpnt1[ii][1];
				point_data1[ii].point[2] = cpnt1[ii][2];
				point_data1[ii].slope_type = UF_CURVE_SLOPE_DIR;
				point_data1[ii].slope[0] = cvect1[ii][0];
				point_data1[ii].slope[1] = cvect1[ii][1];
				point_data1[ii].slope[2] = cvect1[ii][2];
				point_data1[ii].crvatr_type = UF_CURVE_CRVATR_NONE;
			}
			UF_CALL(UF_CURVE_create_spline_thru_pts(degree3, periodicity,
				(no_pts+2),point_data1,NULL,FALSE,&temp_tag));

		}

	goto start1;

ending1:  ;	


}


/*ARGSUSED*/
void ufusr(char *param, int *retcode, int paramLen)
{
    if (UF_CALL(UF_initialize())) return;
    UF_set_retiring_flag( 2 ); /* notify about retiring functions */
    do_it();
    UF_terminate();
}

int ufusr_ask_unload(void)
{
    return (UF_UNLOAD_IMMEDIATELY);
}
